#include <graph/server/gui/mainwindow.h>
#include <graph/server/plugin.h>
#include <graph/server/ui_mainwindow.h>
#include <QFileDialog>
#include <QMessageBox>
#include <iostream>
#include <stdexcept>

namespace Graph::GUI {
MainWindow::MainWindow(QWidget* parent) : QMainWindow(parent), ui(new Ui::MainWindow) {
  ui->setupUi(this);

  // plugins
  pluginsModel = new PluginModel(this);
  ui->pluginsListView->setModel(pluginsModel);

  // unload plugin
  connect(ui->pluginsListView, &QListView::doubleClicked, [&](const QModelIndex& index) {
    pluginsModel->removeRow(index.row());
    pluginsModel->remove(index.row());
    ui->pluginsListView->doItemsLayout();
  });
}

MainWindow::~MainWindow() {
  delete ui;
  delete pluginsModel;
}

void MainWindow::on_action_Add_triggered() {}

void MainWindow::on_action_Load_triggered() {
  QString file = QFileDialog::getOpenFileName(this, "Load plugin", ".", "Plugins (*.so *.dll *.dylib)");

  if (file.isEmpty())
    return;

  try {
    pluginsModel->add(new Plugin(file.toStdString()));
  } catch (std::runtime_error e) {
    std::cout << e.what() << std::endl;
    QMessageBox::critical(this, "Error", "Failed to load plugin.", QMessageBox::StandardButton::Ignore);
  }

  int row = pluginsModel->rowCount();
  pluginsModel->insertRow(row);
  QModelIndex index = pluginsModel->index(row);
  ui->pluginsListView->doItemsLayout();
  ui->pluginsListView->setCurrentIndex(index);
}

} // namespace Graph::GUI
